import unittest
import os
from pdf_gen.pdf_gen import pdf, merge_pdf


def pathCheck(path: str):
    path = pdf._get_path(pdf, path)
    if path.count(':') == 1:
        if path.count('<>?"\*') == 0:
            return True


def fileCompare(example_path: str, compare_path: str):
    ext = 0
    with open(example_path, 'rb') as example, open(compare_path, 'rb') as compare:
        ex = (string for string in example)
        cmp = (string for string in compare)
        for string in ex:
            if string != next(cmp):
                ext += 1
                if ext > 2:
                    return False
        else:
            return True


def create_temp_files(path: str):
    file_names = ['mg1', 'mg2', 'mg3', 'mg4']
    for file_name in file_names:
        page = pdf(path, f'{file_name}.pdf')
        page.write_text(file_name + ' merge_test')
        page.save()


class pdf_test(unittest.TestCase):

    def test_get_path(self):
        paths = ['c:/path', 'c::/path', 'C:/path/pdf.p\df', 'c:/path/pd>f.pdf.pdf.gen', 'c: /path/ ']
        names = ['gen', 'generated', 'gena', 'gener', 'fic']
        for path in paths:
            for name in names:
                self.assertTrue(pathCheck(path))
                self.assertEqual(pdf._get_path(pdf, path, name), f'c:/path/{name}.pdf')

    def test_format_data(self):
        data1 = {
            'title': 'Table title',
            'columns': [
                {'name': 'Name', 'value': 'name'},
                {'name': 'Age', 'value': 'age'}
            ],
            'rows': [
                {'name': 'string1', 'age': 23},
                {'name': 'string2', 'age': 43}
            ]
        }
        ndata1 = [
            ['Table title'],
            ['Name', 'Age'],
            ['string1', 23],
            ['string2', 43]
        ]
        data2 = {
            'title': 'Table title',
            'columns': [
                {'name': 'Name', 'value': 'name'},
                {'name': 'Lastname', 'value': 'lastname'},
                {'name': 'Age', 'value': 'age'}
            ],
            'rows': [
                {'name': 'string1', 'lastname': '1st', 'age': 23},
                {'name': 'string2', 'lastname': '2nd', 'age': 43},
                {'name': 'string3', 'lastname': '3rd', 'age': 35}
            ]
        }
        ndata2 = [
            ['Table title'],
            ['Name', 'Lastname', 'Age'],
            ['string1', '1st', 23],
            ['string2', '2nd', 43],
            ['string3', '3rd', 35]
        ]
        data = [data1, data2]
        ndata = [ndata1, ndata2]
        for id in range(2):
            self.assertEqual(pdf._format_data(pdf, data[id]), ndata[id])

    def test_pdfCreate(self):
        paths = ['c:/projects/pdf_generator/path', 'c::/projects/pdf_generator/path',
                 'C:/projects/pdf_generator/path/pdf.p\df', 'c:/projects/pdf_generator/path/pd>f.pdf.pdf.gen',
                 'c: /projects/pdf_generator/path/ ']
        names = ['gen', 'generated', 'gena', 'gener', 'fic']
        for path in paths:
            for name in names:
                testing = pdf(path, name)
                testing.save()
                self.assertTrue(os.path.exists(testing._get_path(path, name)))
                os.remove(testing._get_path(path, name))

    def test_table_text(self):
        file = pdf('c:/projects/pdf_generator/tests/test_files/', 'compare')
        file.write_text('Testing file', 'right', 350, 800)
        data = {
            'title': 'Table title',
            'columns': [
                {'name': 'Name', 'value': 'name'},
                {'name': 'Age', 'value': 'age'}
            ],
            'rows': [
                {'name': 'string1', 'age': 23},
                {'name': 'string2', 'age': 43}
            ]
        }
        file.draw_table(data)
        file.save()
        self.assertTrue(fileCompare('c:/projects/pdf_generator/tests/test_files/compare.pdf',
                                    'c:/projects/pdf_generator/tests/test_files/test.pdf'))
        os.remove('c:/projects/pdf_generator/tests/test_files/compare.pdf')


class merger_test(unittest.TestCase):

    def test_wo_del(self):
        paths = ['c:/projects/pdf_generator/tests/test_files/mg1.pdf',
                 'c:/projects/pdf_generator/tests/test_files/mg2.pdf',
                 'c:/projects/pdf_generator/tests/test_files/mg3.pdf',
                 'c:/projects/pdf_generator/tests/test_files/mg4.pdf']
        result = 'c:/projects/pdf_generator/tests/test_files/mg_result.pdf'
        compare = 'c:/projects/pdf_generator/tests/test_files/mg_compare.pdf'
        merge_pdf(paths, result)
        self.assertTrue(fileCompare(result, compare))
        os.remove(result)
        for file in paths:
            self.assertTrue(os.path.exists(file))

    def test_del(self):
        paths = ['c:/projects/pdf_generator/path/mg1.pdf',
                 'c:/projects/pdf_generator/path/mg2.pdf',
                 'c:/projects/pdf_generator/path/mg3.pdf',
                 'c:/projects/pdf_generator/path/mg4.pdf']
        create_temp_files('c:/projects/pdf_generator/path')
        result = 'c:/projects/pdf_generator/path/mg_result.pdf'
        compare = 'c:/projects/pdf_generator/tests/test_files/mg_compare.pdf'
        merge_pdf(paths, result, True)
        self.assertTrue(fileCompare(result, compare))
        os.remove(result)
        for file in paths:
            self.assertFalse(os.path.exists(file))



import setuptools
import pdf_gen

with open('Readme.md') as fr:
    long_description = fr.read()

setuptools.setup(
    name='pdf_gen',
    version=pdf_gen.__version__,
    author='Desyatnikov I.S.',
    author_email='ncab_03@mail.ru',
    description='Pdf-file generator',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/vanya2535/pdf_gen',
    packages=setuptools.find_packages(),
    install_requires=['reportlab >= 3.5.65',
                      'PyPDF2 >= 1.26.0'],
    test_suite='tests',
    python_requires='>=3.7',
    platforms=["any"]
)



from reportlab.pdfgen.canvas import Canvas
from reportlab.pdfbase import ttfonts, pdfmetrics
from reportlab.platypus import Table
from random import uniform, randint, choice
from PyPDF2 import PdfFileMerger
import os


class pdf:

    def __init__(self, path: str, name: str = 'generated'):
        """
        Create a pdf-file object\n
        :param path: path to create file
        :param name: name of file
        """
        self.file = Canvas(self._get_path(path, name))
        self.set_font(12)

    def _get_path(self, path: str, name: str = 'generated') -> str:
        """
        This function cleans path\n
        :param path: path to create file
        :param name: name of file
        :return: clean path to file
        """
        path = ''.join(symbol for symbol in path.lower() if symbol not in ' <>?"\*')
        while path.count(':') > 1:
            path = path[:path.rfind(':')] + path[path.rfind(':') + 1:]
        while path[len(path) - 1] == ' ':
            path = path[:len(path) - 1]
        if ".pdf" in path:
            path = path[0:path.rfind('/') + 1:1]
        if path[len(path) - 1] != '/':
            path += '/'
        if '.pdf' in name:
            name = name[:name.rfind('.')]
        return path + name + '.pdf'

    def set_font(self, font_size: int):
        """
        This function set up font and his size in file\n
        :param font_size: size of font
        """
        self.font_size = font_size
        using_font = ttfonts.TTFont("Calibri", "Calibri.ttf")
        pdfmetrics.registerFont(using_font)
        self.file.setFont("Calibri", self.font_size)

    def write_text(self, text: str, position: str = "mid", x: float = 297.635, y: float = 815.89):
        """"
        This function write text on defined position\n
        size of page is 595.27,841.89
        :param text: string of text to writing
        :param position: left/mid/right position of string of text
        :param x, y: coordinates of string
        """
        if position == "left":
            self.file.drawString(x, y, text)
        elif position == "mid":
            self.file.drawString(x, y, text)
        elif position == "right":
            self.file.drawRightString(x, y, text)

    def random_drawing(self, fg_count: int):
        """
        This function draws random picture\n
        :param fg_count: count of figures, drawn on page
        """
        for figure in range(fg_count):
            methods = [
                self.file.bezier(randint(150, 495), randint(150, 741), randint(150, 495), randint(150, 741),
                                 randint(150, 495), randint(150, 741), randint(150, 495), randint(150, 741)),
                self.file.arc(randint(100, 495), randint(100, 741), randint(100, 495), randint(100, 741)),
                self.file.rect(randint(100, 395), randint(100, 641), randint(1, 100), randint(1, 100),
                               fill=randint(0, 1)),
                self.file.ellipse(randint(100, 495), randint(100, 741), randint(100, 495), randint(100, 741),
                                  fill=randint(0, 1)),
                self.file.circle(randint(100, 395), randint(100, 641), randint(1, 100), fill=randint(0, 1)),
                self.file.roundRect(randint(100, 395), randint(100, 641), randint(1, 100), randint(1, 100),
                                    randint(1, 100), fill=randint(0, 1))
            ]
            self.file.setFillColorRGB(uniform(0, 1), uniform(0, 1), uniform(0, 1), alpha=uniform(0, 1))
            self.file.setStrokeColorRGB(uniform(0, 1), uniform(0, 1), uniform(0, 1), alpha=uniform(0, 1))
            choice(methods)

    def _format_data(self, data: dict) -> list:
        """
        This function processing data and return list of data for create table\n
        :param data: dict of data
        :return: list of data
        """
        new_data = [[data['title']]]
        add_list = []
        value_list = []
        for column_elem in data['columns']:
            add_list.append(column_elem['name'])
            value_list.append(column_elem['value'])
        new_data.append(add_list.copy())
        add_list.clear()
        for row_elem in data['rows']:
            for value in value_list:
                add_list.append(row_elem[value])
            new_data.append(add_list.copy())
            add_list.clear()
        return new_data

    def draw_table(self, data: dict):
        """
        This function draws table from your dictionary of data\n
        :param data: dictionary with data, e.g.
        {
            'title': 'Table title',
            'columns': [
                {'name': 'Name', 'value': 'name'},
                {'name': 'Age', 'value': 'age'}
            ],
            'rows': [
                {'name': 'string1', 'age': 23},
                {'name': 'string2', 'age': 43}
            ]
        }
        """
        data = self._format_data(data)
        table = Table(data=data,
                      style=[("GRID", (0, 1), (-1, -1), 1, "Black"),
                             ("FONT", (0, 0), (-1, -1), "Calibri", self.font_size),
                             ("BOX", (0, 0), (-1, -1), 1, "Black")])
        table.wrapOn(self.file, 10, 10)
        table.drawOn(self.file, 10, 10)

    def save(self, author: str = 'pdf_gen', title: str = 'GENERATED'):
        """
        This function saves our file\n
        :param author: author of file
        :param title: title of file
        """
        self.file.setAuthor(author)
        self.file.setTitle(title)
        self.file.save()


def merge_pdf(source_paths: list, result_path: str, source_del: bool = False):
    """
    This function merges some files into one\n
    You can create some pages via "pdf" class and merge it into one pdf-file\n
    :param source_paths: paths to source files, e.g. ['c:/files/file1.pdf', 'c:/files/file2.pdf']
    :param result_path: path to result file
    :param source_del: if it`s True, function deletes source files, else not
    """
    merger = PdfFileMerger()
    opened = []
    for path in source_paths:
        file = open(path, 'rb')
        merger.append(fileobj=file)
        opened.append(file)
    result = open(result_path, 'wb')
    merger.write(result)
    result.close()
    for file in opened:
        file.close()
    if source_del:
        for path in source_paths:
            os.remove(path)
